#==============================================================================
# package      DE_EPAGES::MailType::UI::User
# state        private
#==============================================================================
package DE_EPAGES::MailType::UI::User;
use base qw( DE_EPAGES::Presentation::UI::Login );

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Password qw ( CryptPassword );
use DE_EPAGES::Core::API::PerlTools qw( Compare GetCurrentDateTime );
use DE_EPAGES::Permission::API::Permission qw (ActionByName);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

sub ViewUserChangeLostPassword {
    my $self = shift;
    my $Servlet = shift;
    my $User = $Servlet->object;

    $Servlet->redirectToSecureConnection($User->getSite()->get('WebUrlAdminSSL'));

    my $PwdToken = $Servlet->params('Token');
    my $UserPwdToken = $User->get('PWDToken');
    if (!_isTokenValid($User, $UserPwdToken, $PwdToken)) {
        $Servlet->vars('INVALID_TOKEN', 1);
        $Servlet->vars('LoginUrl', _url($User->getSite()->get('WebUrlAdminSSL'), $Servlet->params('LastViewAction')));
    }
    $Servlet->vars('Token', $PwdToken);
    $Servlet->vars('LastViewAction', $Servlet->params('LastViewAction'));
    return;
}

sub UnityViewUserChangeLostPassword {
    my $self = shift;
    my $Servlet = shift;

    $self->ViewUserChangeLostPassword($Servlet);

    my $ViewAction = ActionByName($Servlet->object->class, $Servlet->vars('ViewAction'));
    $Servlet->vars('Title', $ViewAction->get('NameOrAlias'));
}

sub SaveUserChangedLostPassword {
    my $self = shift;
    my $Servlet = shift;
    my $User = $Servlet->object;

    $Servlet->testSecureConnection();
    $Servlet->testPostRequest();

    my $hValues = $Servlet->form->form($User, 'SaveLostPassword');

    my $PwdToken = $Servlet->form->value('Token');
    my $UserPwdToken = $User->get('PWDToken');
    if (!_isTokenValid($User, $UserPwdToken, $PwdToken)) {
        $Servlet->vars('ViewAction', 'ViewUserChangeLostPassword');
        $Servlet->vars('ViewObjectID', $User->id);
        return;
    }

    my $NewPassword = $hValues->{'NewPassword'} ;
    if ($hValues->{'PasswordConf'} ne $NewPassword) {
        $Servlet->form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'Save',
            'Name'  => 'NewPassword',
        }) ;
        $Servlet->form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'Save',
            'Name'  => 'PasswordConf',
        }) ;
    }
    # Crypt password if new password is submitted.
    my $Password;
    eval { $Password = CryptPassword($NewPassword); };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'NativeDie' and $Error->shortMessage eq 'Wide character in crypt';
        $Servlet->form->addFormError({
            'Reason'=> 'PASSWORD_WRONG',
            'Form'  => 'Save',
            'Name'  => 'NewPassword',
        }) ;
    }
    TriggerHook(
        'UI_BeforeSaveMailTypeLostPassword',
        {
            'Servlet'           => $Servlet,
            'User'              => $User,
            'Password'          => $NewPassword,
            'FormName'          => 'Save',
            'FormField'         => 'NewPassword',
        }
    );
    $Servlet->form->executeFormError;
    $User->set({'Password' => $Password, 'PWDToken' => undef, 'PWDTokenExpireDate' => undef});
    TriggerHook(
        'UI_AfterSaveMailTypeLostPassword',
        {
            'Servlet'           => $Servlet,
            'User'              => $User,
            'Password'          => $NewPassword
        }
    );

    $User->set({'CountLoginFailure' => 0});

    $Servlet->logout(1);
    $self->LoginUser($Servlet, $User);

}

sub _url {
    my ($URL, $ViewAction) = @_;
    if ($URL =~ /\/\?/) {
        $URL .= '&ViewAction='.$ViewAction;
    } else {
        $URL .= '/?ViewAction='.$ViewAction;
    }
}

sub _checkToken {
    my ($User) = @_;
    my $TokenExpireDate = $User->get('PWDTokenExpireDate');
    if (Compare('DateTime', $TokenExpireDate, GetCurrentDateTime()) == -1) {
        $User->set({'PWDToken' => undef, 'PWDTokenExpireDate' => undef});
        return 0;
    }
    return 1;
}

sub _isTokenValid {
    my ($User, $UserPwdToken, $PwdToken) = @_;
    return 0 unless defined $UserPwdToken;
    return 0 unless defined $PwdToken;
    return 0 if $UserPwdToken ne $PwdToken;
    return _checkToken($User);
}

1;
