#========================================================================================
# package      DE_EPAGES::Order::Hooks::Content
# state        public
# description  creates and deletes the withdrawal rights as PDF
#========================================================================================
package DE_EPAGES::Order::Hooks::Content;

use strict;
use DE_EPAGES::Core::API::File qw ( DeleteDirectory ExistsDirectory );
use DE_EPAGES::Order::API::OrderDocument qw (
    GetLegalDocumentPDFLocation
);


#========================================================================================
# function     OnAfterChangeContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterChangeContent($hParams);
#----------------------------------------------------------------------------------------
# description  Deletes the withdrawal rights PDF if the withdrawal page is updated.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ContentObjectID - object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub OnAfterChangeContent {
    my ($hParams) = @_;

    _deleteLegalDocumentPDFLocation($hParams->{'Object'});

    return;
}

#========================================================================================
# function     OnDeleteContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteContent($hParams);
#----------------------------------------------------------------------------------------
# description  Deletes the withdrawal rights PDF if the withdrawal page is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ContentObjectID - object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub OnDeleteContent {
    my ($hParams) = @_;

    _deleteLegalDocumentPDFLocation($hParams->{'Object'});

    return;
}

#========================================================================================
sub _deleteLegalDocumentPDFLocation {
    my ($ContentObject) = @_;
    my $Shop = $ContentObject->getSite();

    return unless ($ContentObject->class->alias eq 'CustomerInformation'
        || ($Shop->isUnity() && $ContentObject->parent->alias eq 'LegalPagesUnity'));

    my $Location = GetLegalDocumentPDFLocation($Shop, $ContentObject);
    DeleteDirectory($Location, { 'recursive' => 1 }) if ExistsDirectory($Location);

    return;
}

1;
