#========================================================================================
# package      DE_EPAGES::Order::UI::OrderDocument
# state        private
#========================================================================================
package DE_EPAGES::Order::UI::OrderDocument;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Object::API::Factory qw (Transaction);
use DE_EPAGES::Order::API::OrderDocument qw ( SetOrderChangeLastUpdatedOn );
#==============================================================================
# Event handlers
#==============================================================================

sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $OrderDocument = $Servlet->object();
    my $Shop = $OrderDocument->getSite();

    #set attributes of particular order document
    if ($OrderDocument->get('Issued')) {
        $Servlet->form->executeFormError({
            'Name' => 'Issued',
            'Reason' => 'IssuedIsSet',
            });
    }
    $self->SUPER::Save($Servlet);

    my $LineItemContainer = $OrderDocument->get('LineItemContainer');
    #update the lineitems for the document
    my $LineItemForm = $Servlet->form->form($OrderDocument, 'Save');
    # update LineItems (set the new quantity and delete the discount if quantity was changed)
    my $Quantities = $LineItemForm->{'LineItems'};
    my $IsChanged = 0;
    foreach my $QuantityEntry (@$Quantities) {
        my $ObjectID = $QuantityEntry->{'LineItemID'};
        delete $QuantityEntry->{'LineItemID'};
        my $LineItem = $Shop->loadObject($ObjectID, 'LineItem');
        die 'invalid LineItem:'.$LineItem->id.' used in OrderDocument:'.$OrderDocument->id
         if $LineItem->get('LineItemContainer')->id != $LineItemContainer->id;
        if ($LineItem->get('Quantity') != $QuantityEntry->{'Quantity'}) {
            $LineItem->set({'Discount'=> undef}) ;
            $LineItem->set($QuantityEntry) ;
            $IsChanged = 1;
        }
    }
    my $LineItemPrices = $LineItemForm->{'DiscountLineItems'};
    foreach my $PriceEntry (@$LineItemPrices) {
        my $ObjectID = $PriceEntry->{'DiscountLineItemID'};
        my $LineItem = $Shop->loadObject($ObjectID, 'LineItem');
        die 'invalid LineItem:'.$LineItem->id.' used in OrderDocument:'.$OrderDocument->id
         if $LineItem->get('LineItemContainer')->id != $LineItemContainer->id;
        if ($LineItem->get('LineItemPrice') != $PriceEntry->{'DiscountPrice'}) {
            $LineItem->set({'LineItemPrice' => $PriceEntry->{'DiscountPrice'}}) ;
            $IsChanged = 1;
        }
    }
    $LineItemContainer->recalculate if $IsChanged;

    return;
}

sub Send {
    my $self = shift;
    my ($Servlet) = @_;
    my $OrderDocument = $Servlet->object();


    # save before send
    $self->Save($Servlet);
    Transaction( sub {
        if (not $OrderDocument->existsChild('LineItemContainer') ) {
            my $Container = $OrderDocument->get('LineItemContainer');
            $Container->clone({'Parent' => $OrderDocument,
             'Alias'  => 'LineItemContainer',
             'Inherit'       => 1,
            },1) if defined $Container;
        }
        if (not $OrderDocument->existsChild('Address') ) {
            my $Address = $OrderDocument->get('Address');
            $Address->clone({'Parent' => $OrderDocument,
             'Alias'  => 'Address',
             'Inherit'       => 1,
            },1) if defined $Address;
        }
        if (not $OrderDocument->existsChild('ShopAddress') ) {
            my $ShopAddress = $OrderDocument->get('ShopAddress');
            $ShopAddress->clone({'Parent' => $OrderDocument,
             'Alias'  => 'ShopAddress',
             'Inherit'       => 1,
            },1) if defined $ShopAddress;
        }
        #set issue date
        $OrderDocument->set({'Issued' => 1});
    });
    
    SetOrderChangeLastUpdatedOn($OrderDocument->get('Order'));
    
    return;
}

1;
