#========================================================================================
# package      DE_EPAGES::Order::t::unit::OrderCollectionResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::Resource::OrderCollection
#========================================================================================
package DE_EPAGES::Order::t::unit::OrderCollectionResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::Mockery;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Object Number HashRef );

use Test::More;

use strict;

use constant PAGE => 1;
use constant RESULTS_PER_PAGE => 10;
use constant SHOP_ID => 123-456;


sub test {
    my $self = shift;

    $self->test_get();

    done_testing();
}



sub test_get {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $OrderCollectionResource = $Mockery->get('SUT');

    my $Shop = $Mockery->get('Shop');
    $Shop->set({'Site' => $Shop});
    my $ResultsPerPage = RESULTS_PER_PAGE;
    my $Page = PAGE;
    my $hQuery = {};

    my $hOrderCollectionData = $OrderCollectionResource->get($Shop, 'en_GB', $ResultsPerPage, $Page, $hQuery);

    is_deeply($hOrderCollectionData, $self->_createOrderCollectionData(), "$SubTestName - test if expected order data was returned");
}

sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['PagerSearch', 'OrderService', 'OrderSearchService'], sub {
        my ($PagerSearch, $OrderService, $OrderSearchService) = @_;
        my $Mockify = Test::Mockify->new('DE_EPAGES::Order::API::Resource::OrderCollection', []);
        
        $Mockify->mock('_createPagerSearch')->when(HashRef())->thenReturn($PagerSearch);
        $Mockify->mock('_createOrderService')->when()->thenReturn($OrderService);
        $Mockify->mock('_createOrderSearchService')->when(Object('DE_EPAGES::Shop::API::Object::Shop'),Number(RESULTS_PER_PAGE), Number(PAGE), HashRef())->thenReturn($OrderSearchService);

        return $Mockify->getMockObject();
    });
    ## no critic (ProhibitMagicNumbers)
    $Mockery->register('Order', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', {
            'Alias' => 'TestOrder',
        });
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('Order2', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', {
            'Alias' => 'AnotherOrder',
        });
        return $MockifyEpagesObject->getMockObject();
    });
    ## use critic (ProhibitMagicNumbers)
    $Mockery->register('Shop', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Alias' => 'TestShop',
            'LocaleID' => 'de_DE',
            'ID' => SHOP_ID,
        });
        $MockifyEpagesObject->addMockWithReturnValueAndParameterCheck('existsSiteLocale', 1, [{'string' => 'en_GB'}]);
        return $MockifyEpagesObject->getMockObject();
    });
    $Mockery->register('OrderService', [], sub {
        my ($OrderService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::OrderService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('getOrderData', $self->_createOrderData(), ['object', 'string']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('OrderSearchService', [], sub {
        my ($OrderService) = @_;
        my $Mockify = Test::Mockify->new('DE_EPAGES::Order::API::OrderSearchService', []);
        $Mockify->mock('ConvertEpagesJValuesToEpages6')->when()->thenReturnUndef();
        $Mockify->mock('DefineSearchParameters')->when()->thenReturnUndef();
        $Mockify->mock('GetSearchParameters')->when()->thenReturn({
            'ObjectID' => SHOP_ID
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('PagerSearch', ['Order', 'Order2'], sub {
        my ($Order, $Order2) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Core::API::Object::BaseObject', []);
        my $PagerSearch = $Mockify->getMockObject();
        $PagerSearch->mock('items', sub {
            return [$Order, $Order2];
        });
        $PagerSearch->{'ItemCount'} = 2;
        return $PagerSearch;
    });

    return $Mockery->assemble();
}



sub _createOrderCollectionData {
    my $self = shift;
    return {
        'page' => PAGE,
        'items' => [$self->_createOrderData(), $self->_createOrderData()],
        'resultsPerPage' => RESULTS_PER_PAGE,
        'results' => 2
    };
}



sub _createOrderData {
    return  {
        'OrderName' => 'AnOrder',
    };
}

__PACKAGE__->runScript();