#========================================================================================
# package      DE_EPAGES::Order::t::unit::OrderSearchService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::OrderSearchService
#========================================================================================
package DE_EPAGES::Order::t::unit::OrderSearchService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Object String );
use DE_EPAGES::WebService::API::Util qw (  ConvertDateForEP6 );

use strict;

use constant PAGE => 1;
use constant RESULTS_PER_PAGE => 10;
use constant SHOP_ID => '123-456';
use constant SEARCH_NAME => 'LastUpdatedSearch';
use constant UPDATED_FROM_DATE => '2016-07-19';
use constant UPDATED_FROM_TIME => 'T12:08:00.000Z';
use constant UPDATED_FROM_DATETIME => UPDATED_FROM_DATE . UPDATED_FROM_TIME;

sub test {
    my $self = shift;

    $self->test_getSearchParameters();

    done_testing();
}

sub test_getSearchParameters {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hQuery = {
        'locale' => [],
        'page' => [PAGE],
        'resultsPerPage' => [RESULTS_PER_PAGE],
        'updatedFrom' => [UPDATED_FROM_DATETIME]
    };

    my $Mockery = $self->_createMocks($hQuery);
    my $SUT = $Mockery->get('SUT');

    my $Shop = $Mockery->get('Shop');
    $Shop->set({'Site' => $Shop});

    $SUT->ConvertEpagesJValuesToEpages6();

    is($SUT->{'SearchName'}, SEARCH_NAME, "$SubTestName - test if expected SearchName was set");
    is($SUT->{'OrderBy'}, 'LastUpdated', "$SubTestName - test if expected OrderBy was set");
    is_deeply(
            $SUT->{'SearchForm'}->{'UpdatedDateFrom'},
            ConvertDateForEP6(UPDATED_FROM_DATETIME),
            "$SubTestName - test if converted date is correct"
    );

    $SUT->DefineSearchParameters();
    my $hSearchParameters = $SUT->GetSearchParameters();

    is(
            $hSearchParameters->{'Parameters'}->{'Search'},
            'LastUpdatedSearch',
            "$SubTestName - test if expected 'Search' Parameter is returned"
    );
    like(
            $hSearchParameters->{'Parameters'}->{'UpdatedDateFrom'},
            qr/@{[UPDATED_FROM_DATE]}/,
            "$SubTestName - test if expected 'updatedFrom' date is returned"
    );
    is($hSearchParameters->{'OrderBy'}, 'LastUpdated', "$SubTestName - test if expected ordering is returned");
}

sub _createMocks {
    my $self = shift;
    my ($hQuery) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Shop', 'Class'], sub {
        my ($Shop, $Class) = @_;
        my $MockifyOrderSearchService = Test::Mockify->new(
                'DE_EPAGES::Order::API::OrderSearchService', [$Shop, RESULTS_PER_PAGE, PAGE, $hQuery]
        );

        $MockifyOrderSearchService->mock('_LoadClassByAlias')->when(String('CustomerOrder'))->thenReturn($Class);
        $MockifyOrderSearchService->mock('_GetSearchDefinition')
                ->when(Object('DE_EPAGES::Object::API::Object::Class'), String(SEARCH_NAME))
                ->thenReturn('SearchDefinition');

        return $MockifyOrderSearchService->getMockObject();
    });

    $Mockery->register('Shop', [], sub {
        my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Alias' => 'TestShop',
            'LocaleID' => 'de_DE',
            'ID' => SHOP_ID,
        });
        $MockifyShop->addMockWithReturnValueAndParameterCheck('existsSiteLocale', 1, [{'string' => 'en_GB'}]);

        return $MockifyShop->getMockObject();
    });

    $Mockery->register('Class', [], sub {
        my $MockifyClass = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {
        });

        return $MockifyClass->getMockObject();
    });

    return $Mockery->assemble();
}

__PACKAGE__->runScript();