#========================================================================================
# package      DE_EPAGES::Search::DAL::SQL::PagerSearch
# state        private
# base         DE_EPAGES::Search::DAL::SQL::PagerSearch
# description
#========================================================================================
package DE_EPAGES::Search::DAL::SQL::PagerSearch;

use strict;

use DE_EPAGES::Core::API::EpagesConfig qw (EpagesConfig);
use DE_EPAGES::Core::API::Log qw (LogDebug);

sub ExecutePagerSearch {
    my $self = shift;
    my ($dbi, $PageNumber, $SQL, $ParameterValues) = @_;

    my $IsolationLevel = EpagesConfig->get('DE_EPAGES::Search', 'SearchIsolationLevel', 'READ UNCOMMITTED');
    my $hValues = {};
    LogDebug('Isolation Level', $IsolationLevel);
    $dbi->atIsolationLevel($IsolationLevel , sub {
        $hValues->{'CurrentPage'} = $PageNumber;
        $hValues->{'Items'} = [ map {$_->[0]} @{$dbi->execute($SQL, $ParameterValues)} ];
        $hValues->{'ItemCount'} = scalar @{$hValues->{'Items'}};
    });

    return $hValues;
}

sub GetSQLSelect {
    return 'SELECT DISTINCT %s.%s';
}

sub GetSQLSimpleSelect {
    return 'SELECT %s.%s';
}

sub GetSQLLimit {
    my $self = shift;
    my ($dbi, $Offset, $PageSize) = @_;
    return '';
}

sub GetSQLGroup {
    my $self = shift;

    return 'GROUP BY %s.%s';
}

sub GetSQLSort {
    my $self = shift;
    my ($dbi, $OrderByPart, $OrderDesc, $IgnoreCase) = @_;

    return 'ORDER BY '. $OrderByPart
        . ($OrderDesc ? ' DESC' : '');
}

sub GetSQLSortMax {
    my $self = shift;
    my ($dbi, $OrderByPart, $OrderDesc, $IgnoreCase) = @_;

    return "ORDER BY MAX($OrderByPart)" . ($OrderDesc ? ' DESC' : '');
}
1;
