#========================================================================================
# package      DE_EPAGES::Search::DAL::mysql::PagerSearch
# state        private
# description
#========================================================================================
package DE_EPAGES::Search::DAL::mysql::PagerSearch;
use base qw (Exporter);
use strict;

use DE_EPAGES::Core::API::EpagesConfig qw (EpagesConfig);
use DE_EPAGES::Core::API::Log qw (LogDebug);

sub ExecutePagerSearch {
    my $self = shift;
    my ($dbi, $PageNumber, $SQL, $ParameterValues) = @_;

    my $hValues = {};

    my $SearchQuery = sub {
        $hValues->{'CurrentPage'} = $PageNumber;
        $hValues->{'Items'} = [ map {$_->[0]} @{$dbi->execute($SQL, $ParameterValues)} ];
        $hValues->{'ItemCount'} = $dbi->execute('SELECT FOUND_ROWS()')->[0]->[0];
    };

    unless ($dbi->inTransaction) {
        my $IsolationLevel = EpagesConfig->get('DE_EPAGES::Search', 'SearchIsolationLevel', 'READ UNCOMMITTED');
        LogDebug('Isolation Level', $IsolationLevel);
        $dbi->atIsolationLevel($IsolationLevel, $SearchQuery);
    } else {
       $SearchQuery->(); 
    }

    return $hValues;
}

sub GetSQLSelect {
    return 'SELECT SQL_CALC_FOUND_ROWS DISTINCT %s.%s';
}

sub GetSQLSimpleSelect {
    return 'SELECT SQL_CALC_FOUND_ROWS %s.%s';
}

sub GetSQLLimit {
    my $self = shift;
    my ($dbi, $Offset, $PageSize) = @_;
    return 'LIMIT ' . $Offset . ', ' . $PageSize ;
}

sub GetSQLGroup {
    my $self = shift;

    return 'GROUP BY %s.%s';
}

sub GetSQLSort {
    my $self = shift;
    my ($dbi, $OrderByPart, $OrderDesc, $IgnoreCase) = @_;

    return 'ORDER BY '. $OrderByPart
        . ($IgnoreCase ? ' COLLATE utf8_general_ci' : '')
        . ($OrderDesc ? ' DESC' : '');
}

sub GetSQLSortMax {
    my $self = shift;
    my ($dbi, $OrderByPart, $OrderDesc, $IgnoreCase) = @_;

    return "ORDER BY MAX($OrderByPart)"
        . ($IgnoreCase ? ' COLLATE utf8_general_ci' : '')
        . ($OrderDesc ? ' DESC' : '');
}

1;
