#==============================================================================
# package      DE_EPAGES::ShopMailType::UI::User
# state        private
#==============================================================================
package DE_EPAGES::ShopMailType::UI::User;
use base DE_EPAGES::Presentation::UI::Login;

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Password qw( CryptPassword );
use DE_EPAGES::Core::API::PerlTools qw( Compare GetCurrentDateTime trim);
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );

#==============================================================================
# Event handlers
#==============================================================================

sub ViewChangeLostPassword {
    my $self = shift;
    my $Servlet = shift;
    my $User = $Servlet->object;

    $Servlet->redirectToSecureConnection($User->getSite()->get('WebServerScriptUrlSSL'));

    $Servlet->setLocale($User->get('LocaleID'));

    my $PwdToken = $Servlet->params('Token');
    my $UserPwdToken = $User->get('PWDToken');
    if (!_isTokenValid($User, $UserPwdToken, $PwdToken)) {
        $Servlet->vars('INVALID_TOKEN', 1);
    }
    $Servlet->vars('Token', $PwdToken);
    return;
}

sub SaveChangedLostPassword {
    my $self = shift;
    my $Servlet = shift;
    my $User = $Servlet->object;

    $Servlet->testSecureConnection();
    $Servlet->testPostRequest();

    my $hValues = $Servlet->form->form($User, 'SaveLostPassword');

    my $PwdToken = $Servlet->form->value('Token');
    my $UserPwdToken = $User->get('PWDToken');
    if (!_isTokenValid($User, $UserPwdToken, $PwdToken)) {
        $Servlet->vars('ViewAction', 'ViewChangeLostPassword');
        $Servlet->vars('ViewObjectID', $User->id);
        return;
    }

    my $NewPassword = $hValues->{'NewPassword'} ;
    if ($hValues->{'PasswordConf'} ne $NewPassword) {
        $Servlet->form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'Save',
            'Name'  => 'NewPassword',
        }) ;
        $Servlet->form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'Save',
            'Name'  => 'PasswordConf',
        }) ;
    }

    my ($TrimmedPassword) = trim($NewPassword);
    if ($NewPassword ne $TrimmedPassword) {
        $Servlet->form->addFormError({
            'Reason'=> 'TRAILING_SPACES',
            'Form'  => 'Save',
            'Name'  => 'NewPassword',
        }) ;
    }
    # Crypt password if new password submitted.
    my $Password;
    eval { $Password = CryptPassword($NewPassword); };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'NativeDie' and $Error->shortMessage eq 'Wide character in crypt';
        $Servlet->form->addFormError({
            'Reason'=> 'PASSWORD_WRONG',
            'Form'  => 'Save',
            'Name'  => 'NewPassword',
        }) ;
    }
    TriggerHook(
        'UI_BeforeSaveShopMailTypeLostPassword',
        {
            'Servlet'           => $Servlet,
            'User'              => $User,
            'Password'          => $NewPassword,
            'FormName'          => 'Save',
            'FormField'         => 'NewPassword',
        }
    );
    $Servlet->form->executeFormError;
    $User->set({'Password' => $Password, 'PWDToken' => undef, 'PWDTokenExpireDate' => undef});
    TriggerHook(
        'UI_AfterSaveShopMailTypeLostPassword',
        {
            'Servlet'           => $Servlet,
            'User'              => $User,
            'Password'          => $NewPassword
        }
    );

    $Servlet->vars('SavedPassword', 1);

    $Servlet->logout(1);
    $self->LoginUser($Servlet, $User);
}

sub _checkToken {
    my ($User) = @_;
    my $TokenExpireDate = $User->get('PWDTokenExpireDate');
    if (Compare('DateTime', $TokenExpireDate, GetCurrentDateTime()) == -1) {
        $User->set({'PWDToken' => undef, 'PWDTokenExpireDate' => undef});
        return 0;
    }
    return 1;
}

sub _isTokenValid {
    my ($User, $UserPwdToken, $PwdToken) = @_;
    return 0 unless defined $UserPwdToken;
    return 0 unless defined $PwdToken;
    return 0 if $UserPwdToken ne $PwdToken;
    return _checkToken($User);
}

1;
